import { JsonCard } from "@/components/cards/JsonCard";
import { MessageContextCard } from "@/components/cards/MessageContextCard";
import { ReportCard } from "@/components/cards/ReportCard";
import { ServerCard } from "@/components/cards/ServerCard";
import { UserCard } from "@/components/cards/UserCard";
import { NavigationToolbar } from "@/components/common/NavigationToolbar";
import { Badge } from "@/components/ui/badge";
import { Button, buttonVariants } from "@/components/ui/button";
import {
  Card,
  CardDescription,
  CardHeader,
  CardTitle,
} from "@/components/ui/card";
import {
  DropdownMenu,
  DropdownMenuContent,
  DropdownMenuItem,
  DropdownMenuLabel,
  DropdownMenuSeparator,
  DropdownMenuTrigger,
} from "@/components/ui/dropdown-menu";
import { Separator } from "@/components/ui/separator";
import { Textarea } from "@/components/ui/textarea";
import {
  fetchReportById,
  fetchReports,
  fetchSnapshotsByReport,
  fetchUserById,
} from "@/lib/db";
import Link from "next/link";
import { notFound } from "next/navigation";
import { Fragment } from "react";

export default async function Reports({ params }: { params: { id: string } }) {
  const report = await fetchReportById(params.id);
  if (!report) return notFound();

  const author = await fetchUserById(report.author_id);
  const snapshots = await fetchSnapshotsByReport(report._id);

  const relatedReports = await Promise.all(
    snapshots
      .map((snapshot) => snapshot._id)
      .map((contentId) =>
        fetchReports({
          _id: {
            $ne: report._id,
          },
          status: "Created",
          "content.id": contentId,
        })
      )
  );

  return (
    <div className="flex flex-col gap-2">
      <NavigationToolbar>Viewing Report</NavigationToolbar>

      <Card className="shadow-none">
        <CardHeader>
          <CardTitle className="overflow-ellipsis whitespace-nowrap overflow-x-clip flex gap-2 items-center">
            {report.content.report_reason.includes("Illegal") && (
              <Badge variant="destructive">Urgent</Badge>
            )}{" "}
            {report.additional_context || "No reason specified"}
          </CardTitle>
          <CardDescription>
            {report._id.toString().substring(20, 26)} &middot;{" "}
            {report.content.report_reason} &middot; {report.content.type}
          </CardDescription>
        </CardHeader>
      </Card>

      <Textarea
        placeholder="Enter notes here..."
        className="!min-h-0 !h-[76px]"
      />

      <div className="flex gap-2">
        <Button className="flex-1 bg-green-400 hover:bg-green-300">
          Resolve
        </Button>
        <DropdownMenu>
          <DropdownMenuTrigger asChild>
            <Button className="flex-1 bg-red-400 hover:bg-red-300">
              Reject
            </Button>
          </DropdownMenuTrigger>
          <DropdownMenuContent>
            <DropdownMenuItem>Custom Reason</DropdownMenuItem>
            <DropdownMenuSeparator />
            <DropdownMenuLabel>Presets</DropdownMenuLabel>
            <DropdownMenuItem>Invalid</DropdownMenuItem>
            <DropdownMenuItem>False or Spam</DropdownMenuItem>
            <DropdownMenuItem>Duplicate</DropdownMenuItem>
            <DropdownMenuItem>Not Enough Evidence</DropdownMenuItem>
            <DropdownMenuItem>Request Clarification</DropdownMenuItem>
            <DropdownMenuItem>Acknowledge Only</DropdownMenuItem>
            <DropdownMenuItem>Ignore</DropdownMenuItem>
          </DropdownMenuContent>
        </DropdownMenu>
      </div>

      <Link href={`/panel/inspect/user/${author!._id}`}>
        <UserCard user={author!} subtitle="Report Author" />
      </Link>

      {snapshots.map((snapshot, index) => (
        <Fragment key={snapshot._id}>
          <Separator className="my-2" />
          <div className="flex gap-2">
            <div className="flex-[2]">
              {relatedReports[index].length ? (
                <h2 className="text-md pb-2">&nbsp;</h2>
              ) : undefined}
              {snapshot._type === "Message" && (
                <MessageContextCard snapshot={snapshot} />
              )}
              {snapshot._type === "User" && (
                <Link href={`/panel/inspect/user/${snapshot!._id}`}>
                  <UserCard user={snapshot} subtitle="Reported User" />
                </Link>
              )}
              {snapshot._type === "Server" && (
                <Link href={`/panel/inspect/server/${snapshot!._id}`}>
                  <ServerCard server={snapshot} subtitle="Reported Server" />
                </Link>
              )}
            </div>
            {relatedReports[index].length ? (
              <div className="flex-1 min-w-0">
                <h2 className="text-md text-center pb-2">Other Reports</h2>
                {relatedReports[index].map((relatedReport) => (
                  <ReportCard key={relatedReport._id} report={relatedReport} />
                ))}
              </div>
            ) : undefined}
          </div>
        </Fragment>
      ))}

      <Separator className="my-2" />
      <JsonCard obj={snapshots} />
    </div>
  );
}
